@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -69,11 +69,10 @@ package RT::Queues;
 use strict;
 use warnings;
 
+use base 'RT::SearchBuilder';
 
 use RT::Queue;
 
-use base 'RT::SearchBuilder';
-
 sub Table { 'Queues'}
 
 # {{{ sub _Init
@@ -81,10 +80,19 @@ sub _Init {
   my $self = shift;
   $self->{'with_disabled_column'} = 1;
 
-  # By default, order by name
-  $self->OrderBy( ALIAS => 'main',
-		  FIELD => 'Name',
-		  ORDER => 'ASC');
+  # By default, order by SortOrder, then Name
+  $self->OrderByCols(
+      {
+          ALIAS => 'main',
+          FIELD => 'SortOrder',
+          ORDER => 'ASC',
+      },
+      {
+          ALIAS => 'main',
+          FIELD => 'Name',
+          ORDER => 'ASC',
+      }
+  );
 
   return ($self->SUPER::_Init(@_));
 }
@@ -92,7 +100,7 @@ sub _Init {
 sub Limit  {
   my $self = shift;
   my %args = ( ENTRYAGGREGATOR => 'AND',
-	       @_);
+               @_);
   $self->SUPER::Limit(%args);
 }
 
@@ -113,19 +121,13 @@ sub AddRecord {
     $self->{'rows'}++;
 }
 
-
-
-
-=head2 NewItem
-
-Returns an empty new RT::Queue item
-
-=cut
-
-sub NewItem {
+# no need to order here, it's already ordered in _Init
+sub ItemsOrderBy {
     my $self = shift;
-    return(RT::Queue->new($self->CurrentUser));
+    my $items = shift;
+    return $items;
 }
+
 RT::Base->_ImportOverlays();
 
 1;
