@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -67,12 +67,12 @@
 
 package RT::SavedSearches;
 
-use RT::SavedSearch;
-
 use strict;
 use warnings;
 use base 'RT::SharedSettings';
 
+use RT::SavedSearch;
+
 sub RecordClass {
     return 'RT::SavedSearch';
 }
@@ -109,15 +109,6 @@ sub LimitToPrivacy {
     }
 }
 
-### Internal methods
-
-sub _PrivacyObjects {
-    my $self = shift;
-    Carp::carp("RT::SavedSearches->_PrivacyObjects is deprecated. Please use RT::SavedSearch->_PrivacyObjects");
-    my $search = RT::SavedSearch->new($self->CurrentUser);
-    return $search->_PrivacyObjects(@_);
-}
-
 RT::Base->_ImportOverlays();
 
 1;
